@@ -749,6 +749,12 @@ lwgeom_intersection_prec(const LWGEOM* geom1, const LWGEOM* geom2, double prec)
 
 LWGEOM*
 lwgeom_linemerge(const LWGEOM* geom)
+{
+	return lwgeom_linemerge_directed(geom, LW_FALSE);
+}
+
+LWGEOM*
+lwgeom_linemerge_directed(const LWGEOM* geom, int directed)
 {
 	LWGEOM* result;
 	int32_t srid = RESULT_SRID(geom);
@@ -765,7 +771,20 @@ lwgeom_linemerge(const LWGEOM* geom)
 
 	if (!(g1 = LWGEOM2GEOS(geom, AUTOFIX))) GEOS_FAIL();
 
-	g3 = GEOSLineMerge(g1);
+	if (directed)
+	{
+#if POSTGIS_GEOS_VERSION < 31100
+		lwerror("Directed line merging requires GEOS-3.11 or higher");
+		GEOS_FREE_AND_FAIL(g1);
+		return NULL;
+#else
+		g3 = GEOSLineMergeDirected(g1);
+#endif
+	}
+	else
+	{
+		g3 = GEOSLineMerge(g1);
+	}
 
 	if (!g3) GEOS_FREE_AND_FAIL(g1);
 	GEOSSetSRID(g3, srid);
@@ -1695,8 +1714,9 @@ lwpoly_to_points(const LWPOLY* lwpoly, uint32_t npoints, int32_t seed)
 		return NULL;
 	}
 
-	/* Gross up our test set a bit to increase odds of getting coverage in one pass */
-	sample_npoints = npoints * bbox_area / area;
+	/* Gross up our test set a bit (but not too much) to increase
+	 * odds of getting coverage in one pass */
+	sample_npoints = npoints * FP_MIN(bbox_area / area, 10000.0);
 
 	/* We're going to generate points using a sample grid as described
 	 * http://lin-ear-th-inking.blogspot.ca/2010/05/more-random-points-in-jts.html to try and get a more uniform
@@ -1996,6 +2016,7 @@ lwgeom_delaunay_triangulation(const LWGEOM* geom, double tolerance, int32_t outp
 	return result;
 }
 
+
 static GEOSCoordSequence*
 lwgeom_get_geos_coordseq_2d(const LWGEOM* g, uint32_t num_points)
 {
@@ -2090,3 +2111,94 @@ lwgeom_voronoi_diagram(const LWGEOM* g, const GBOX* env, double tolerance, int o
 
 	return lwgeom_result;
 }
+
+
+#if POSTGIS_GEOS_VERSION >= 31100
+LWGEOM*
+lwgeom_concavehull(const LWGEOM* geom, double ratio, uint32_t allow_holes)
+{
+	LWGEOM* result;
+	int32_t srid = RESULT_SRID(geom);
+	uint8_t is3d = FLAGS_GET_Z(geom->flags);
+	GEOSGeometry *g1, *g3;
+	int geosGeomType;
+
+	initGEOS(lwnotice, lwgeom_geos_error);
+
+	if (!(g1 = LWGEOM2GEOS(geom, AUTOFIX))) GEOS_FAIL();
+
+	geosGeomType = GEOSGeomTypeId(g1);
+	if (geosGeomType == GEOS_POLYGON || geosGeomType == GEOS_MULTIPOLYGON) {
+		int is_tight = LW_FALSE;
+		g3 = GEOSConcaveHullOfPolygons(g1, ratio, is_tight, allow_holes);
+	}
+	else {
+		g3 = GEOSConcaveHull(g1, ratio, allow_holes);
+	}
+
+	if (!g3)
+		GEOS_FREE_AND_FAIL(g1);
+
+	GEOSSetSRID(g3, srid);
+
+	if (!(result = GEOS2LWGEOM(g3, is3d)))
+		GEOS_FREE_AND_FAIL(g1, g3);
+
+	GEOS_FREE(g1, g3);
+	return result;
+}
+
+LWGEOM*
+lwgeom_simplify_polygonal(const LWGEOM* geom, double vertex_fraction, uint32_t is_outer)
+{
+	LWGEOM* result;
+	int32_t srid = RESULT_SRID(geom);
+	uint8_t is3d = FLAGS_GET_Z(geom->flags);
+	GEOSGeometry *g1, *g3;
+
+	initGEOS(lwnotice, lwgeom_geos_error);
+
+	if (!(g1 = LWGEOM2GEOS(geom, AUTOFIX))) GEOS_FAIL();
+
+	g3 = GEOSPolygonHullSimplify(g1, is_outer, vertex_fraction);
+
+	if (!g3)
+		GEOS_FREE_AND_FAIL(g1);
+
+	GEOSSetSRID(g3, srid);
+
+	if (!(result = GEOS2LWGEOM(g3, is3d)))
+		GEOS_FREE_AND_FAIL(g1, g3);
+
+	GEOS_FREE(g1, g3);
+	return result;
+}
+
+LWGEOM*
+lwgeom_triangulate_polygon(const LWGEOM* geom)
+{
+	LWGEOM* result;
+	int32_t srid = RESULT_SRID(geom);
+	uint8_t is3d = FLAGS_GET_Z(geom->flags);
+	GEOSGeometry *g1, *g3;
+
+	if (srid == SRID_INVALID) return NULL;
+
+	initGEOS(lwnotice, lwgeom_geos_error);
+
+	if (!(g1 = LWGEOM2GEOS(geom, AUTOFIX))) GEOS_FAIL();
+
+	/* if output != 1 we want polys */
+	g3 = GEOSConstrainedDelaunayTriangulation(g1);
+
+	if (!g3) GEOS_FREE_AND_FAIL(g1);
+	GEOSSetSRID(g3, srid);
+
+	if (!(result = GEOS2LWGEOM(g3, is3d)))
+		GEOS_FREE_AND_FAIL(g1, g3);
+
+	GEOS_FREE(g1, g3);
+	return result;
+}
+
+#endif
