@@ -33,7 +33,6 @@
 #include "lwgeom_rtree.h"
 #include "lwgeom_functions_analytic.h"
 
-
 #include "access/htup_details.h"
 
 /* Prototypes */
@@ -64,33 +63,32 @@ static int point_in_ring_rtree(RTREE_NODE *root, const POINT2D *point);
 PG_FUNCTION_INFO_V1(LWGEOM_simplify2d);
 Datum LWGEOM_simplify2d(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P_COPY(0);
 	double dist = PG_GETARG_FLOAT8(1);
 	GSERIALIZED *result;
 	int type = gserialized_get_type(geom);
-	LWGEOM *in, *out;
+	LWGEOM *in;
 	bool preserve_collapsed = false;
-
-	/* Handle optional argument to preserve collapsed features */
-	if ((PG_NARGS() > 2) && (!PG_ARGISNULL(2)))
-		preserve_collapsed = PG_GETARG_BOOL(2);
+	int modified = LW_FALSE;
 
 	/* Can't simplify points! */
 	if ( type == POINTTYPE || type == MULTIPOINTTYPE )
 		PG_RETURN_POINTER(geom);
 
+	/* Handle optional argument to preserve collapsed features */
+	if ((PG_NARGS() > 2) && (!PG_ARGISNULL(2)))
+		preserve_collapsed = PG_GETARG_BOOL(2);
+
 	in = lwgeom_from_gserialized(geom);
 
-	out = lwgeom_simplify(in, dist, preserve_collapsed);
-	if ( ! out ) PG_RETURN_NULL();
+	modified = lwgeom_simplify_in_place(in, dist, preserve_collapsed);
+	if (!modified)
+		PG_RETURN_POINTER(geom);
 
-	/* COMPUTE_BBOX TAINTING */
-	if (in->bbox)
-		lwgeom_refresh_bbox(out);
+	if (!in || lwgeom_is_empty(in))
+		PG_RETURN_NULL();
 
-	result = geometry_serialize(out);
-	lwgeom_free(out);
-	PG_FREE_IF_COPY(geom, 0);
+	result = geometry_serialize(in);
 	PG_RETURN_POINTER(result);
 }
 
@@ -187,7 +185,7 @@ Datum LWGEOM_line_interpolate_point(PG_FUNCTION_ARGS)
 	GSERIALIZED *result;
 	double distance_fraction = PG_GETARG_FLOAT8(1);
 	int repeat = PG_NARGS() > 2 && PG_GETARG_BOOL(2);
-	int srid = gserialized_get_srid(gser);
+	int32_t srid = gserialized_get_srid(gser);
 	LWLINE* lwline;
 	LWGEOM* lwresult;
 	POINTARRAY* opa;
@@ -224,6 +222,50 @@ Datum LWGEOM_line_interpolate_point(PG_FUNCTION_ARGS)
 
 	PG_RETURN_POINTER(result);
 }
+
+/***********************************************************************
+ * Interpolate a point along a line 3D version
+ * --vincent.mora@oslandia.com;
+ ***********************************************************************/
+
+Datum ST_3DLineInterpolatePoint(PG_FUNCTION_ARGS);
+
+PG_FUNCTION_INFO_V1(ST_3DLineInterpolatePoint);
+Datum ST_3DLineInterpolatePoint(PG_FUNCTION_ARGS)
+{
+	GSERIALIZED *gser = PG_GETARG_GSERIALIZED_P(0);
+	GSERIALIZED *result;
+	double distance = PG_GETARG_FLOAT8(1);
+	LWLINE *line;
+	LWGEOM *geom;
+	LWPOINT *point;
+
+	if (distance < 0 || distance > 1)
+	{
+		elog(ERROR, "line_interpolate_point: 2nd arg isn't within [0,1]");
+		PG_RETURN_NULL();
+	}
+
+	if (gserialized_get_type(gser) != LINETYPE)
+	{
+		elog(ERROR, "line_interpolate_point: 1st arg isn't a line");
+		PG_RETURN_NULL();
+	}
+
+	geom = lwgeom_from_gserialized(gser);
+	line = lwgeom_as_lwline(geom);
+
+	point = lwline_interpolate_point_3d(line, distance);
+
+	lwgeom_free(geom);
+	PG_FREE_IF_COPY(gser, 0);
+
+	result = geometry_serialize(lwpoint_as_lwgeom(point));
+	lwpoint_free(point);
+
+	PG_RETURN_POINTER(result);
+}
+
 /***********************************************************************
  * --jsunday@rochgrp.com;
  ***********************************************************************/
@@ -389,10 +431,8 @@ Datum LWGEOM_snaptogrid_pointoff(PG_FUNCTION_ARGS)
 	getPoint4d_p(in_lwpoint->point, 0, &offsetpoint);
 	grid.ipx = offsetpoint.x;
 	grid.ipy = offsetpoint.y;
-	if (FLAGS_GET_Z(in_lwpoint->flags) ) grid.ipz = offsetpoint.z;
-	else grid.ipz=0;
-	if (FLAGS_GET_M(in_lwpoint->flags) ) grid.ipm = offsetpoint.m;
-	else grid.ipm=0;
+	grid.ipz = lwgeom_has_z((LWGEOM*)in_lwpoint) ? offsetpoint.z : 0;
+	grid.ipm = lwgeom_has_m((LWGEOM*)in_lwpoint) ? offsetpoint.m : 0;
 
 #if POSTGIS_DEBUG_LEVEL >= 4
 	grid_print(&grid);
@@ -440,7 +480,7 @@ Datum ST_LineCrossingDirection(PG_FUNCTION_ARGS)
 	GSERIALIZED *geom1 = PG_GETARG_GSERIALIZED_P(0);
 	GSERIALIZED *geom2 = PG_GETARG_GSERIALIZED_P(1);
 
-	error_if_srid_mismatch(gserialized_get_srid(geom1), gserialized_get_srid(geom2));
+	gserialized_error_if_srid_mismatch(geom1, geom2, __func__);
 
 	type1 = gserialized_get_type(geom1);
 	type2 = gserialized_get_type(geom2);
@@ -896,6 +936,9 @@ int point_in_multipolygon_rtree(RTREE_NODE **root, int polyCount, int *ringCount
 
 	POSTGIS_DEBUGF(2, "point_in_multipolygon_rtree called for %p %d %p.", root, polyCount, point);
 
+	/* empty is not within anything */
+	if (lwpoint_is_empty(point)) return -1;
+
 	getPoint2d_p(point->point, 0, &pt);
 	/* assume bbox short-circuit has already been attempted */
 
@@ -904,6 +947,9 @@ int point_in_multipolygon_rtree(RTREE_NODE **root, int polyCount, int *ringCount
 	/* is the point inside any of the sub-polygons? */
 	for ( p = 0; p < polyCount; p++ )
 	{
+		/* Skip empty polygons */
+		if( ringCounts[p] == 0 ) continue;
+
 		in_ring = point_in_ring_rtree(root[i], &pt);
 		POSTGIS_DEBUGF(4, "point_in_multipolygon_rtree: exterior ring (%d), point_in_ring returned %d", p, in_ring);
 		if ( in_ring == -1 ) /* outside the exterior ring */
@@ -1005,6 +1051,9 @@ int point_in_multipolygon(LWMPOLY *mpolygon, LWPOINT *point)
 
 	POSTGIS_DEBUG(2, "point_in_polygon called.");
 
+	/* empty is not within anything */
+	if (lwpoint_is_empty(point)) return -1;
+
 	getPoint2d_p(point->point, 0, &pt);
 	/* assume bbox short-circuit has already been attempted */
 
