@@ -32,7 +32,7 @@
 #include "lwgeom_log.h"
 #include "liblwgeom.h"
 
-
+#if POSTGIS_PGSQL_VERSION < 180
 typedef enum					/* type categories for datum_to_json */
 {
 	JSONTYPE_NULL,				/* null, so we didn't bother to identify */
@@ -47,6 +47,7 @@ typedef enum					/* type categories for datum_to_json */
 	JSONTYPE_CAST,				/* something with an explicit cast to JSON */
 	JSONTYPE_OTHER				/* all else */
 } JsonTypeCategory;
+#endif
 
 static void array_dim_to_json(StringInfo result, int dim, int ndims, int *dims,
 				  Datum *vals, bool *nulls, int *valcount,
@@ -54,18 +55,24 @@ static void array_dim_to_json(StringInfo result, int dim, int ndims, int *dims,
 				  bool use_line_feeds);
 static void array_to_json_internal(Datum array, StringInfo result,
 								   bool use_line_feeds);
-static void composite_to_geojson(Datum composite,
-					 char *geom_column_name, int32 maxdecimaldigits,
-					 StringInfo result, bool use_line_feeds,
-					 Oid geom_oid, Oid geog_oid);
+static void composite_to_geojson(FunctionCallInfo fcinfo,
+				 Datum composite,
+				 char *geom_column_name,
+				 int32 maxdecimaldigits,
+				 StringInfo result,
+				 bool use_line_feeds,
+				 Oid geom_oid,
+				 Oid geog_oid);
 static void composite_to_json(Datum composite, StringInfo result,
 							  bool use_line_feeds);
 static void datum_to_json(Datum val, bool is_null, StringInfo result,
 						  JsonTypeCategory tcategory, Oid outfuncoid,
 						  bool key_scalar);
+#if POSTGIS_PGSQL_VERSION < 180
 static void json_categorize_type(Oid typoid,
 								 JsonTypeCategory *tcategory,
 								 Oid *outfuncoid);
+#endif
 static char * postgis_JsonEncodeDateTime(char *buf, Datum value, Oid typid);
 static int postgis_time2tm(TimeADT time, struct pg_tm *tm, fsec_t *fsec);
 static int postgis_timetz2tm(TimeTzADT *time, struct pg_tm *tm, fsec_t *fsec, int *tzp);
@@ -89,7 +96,8 @@ ST_AsGeoJsonRow(PG_FUNCTION_ARGS)
 	Oid geom_oid = InvalidOid;
 	Oid geog_oid = InvalidOid;
 
-	postgis_initialize_cache(fcinfo);
+	/* We need to initialize the internal cache to access it later via postgis_oid() */
+	postgis_initialize_cache();
 	geom_oid = postgis_oid(GEOMETRYOID);
 	geog_oid = postgis_oid(GEOGRAPHYOID);
 
@@ -98,7 +106,7 @@ ST_AsGeoJsonRow(PG_FUNCTION_ARGS)
 
 	result = makeStringInfo();
 
-	composite_to_geojson(array, geom_column, maxdecimaldigits, result, do_pretty, geom_oid, geog_oid);
+	composite_to_geojson(fcinfo, array, geom_column, maxdecimaldigits, result, do_pretty, geom_oid, geog_oid);
 
 	PG_RETURN_TEXT_P(cstring_to_text_with_len(result->data, result->len));
 }
@@ -107,8 +115,14 @@ ST_AsGeoJsonRow(PG_FUNCTION_ARGS)
  * Turn a composite / record into GEOJSON.
  */
 static void
-composite_to_geojson(Datum composite, char *geom_column_name, int32 maxdecimaldigits,
-					 StringInfo result, bool use_line_feeds, Oid geom_oid, Oid geog_oid)
+composite_to_geojson(FunctionCallInfo fcinfo,
+		     Datum composite,
+		     char *geom_column_name,
+		     int32 maxdecimaldigits,
+		     StringInfo result,
+		     bool use_line_feeds,
+		     Oid geom_oid,
+		     Oid geog_oid)
 {
 	HeapTupleHeader td;
 	Oid			tupType;
@@ -166,8 +180,14 @@ composite_to_geojson(Datum composite, char *geom_column_name, int32 maxdecimaldi
 			val = heap_getattr(tuple, i + 1, tupdesc, &isnull);
 			if (!isnull)
 			{
-				appendStringInfo(result, "%s",
-					TextDatumGetCString(DirectFunctionCall2(LWGEOM_asGeoJson, val, Int32GetDatum(maxdecimaldigits))));
+				appendStringInfo(
+				    result,
+				    "%s",
+				    TextDatumGetCString(CallerFInfoFunctionCall2(LWGEOM_asGeoJson,
+										 fcinfo->flinfo,
+										 InvalidOid,
+										 val,
+										 Int32GetDatum(maxdecimaldigits))));
 			}
 			else
 			{
@@ -214,6 +234,7 @@ composite_to_geojson(Datum composite, char *geom_column_name, int32 maxdecimaldi
  * json.c from the Postgres source tree as of 2019-03-28.
  * It would be far better if these were exported from the
  * backend so we could just use them here. Maybe someday.
+ * Sequel: 2022-04-04 That some day finally came in PG15
  */
 
 /*
@@ -223,6 +244,7 @@ composite_to_geojson(Datum composite, char *geom_column_name, int32 maxdecimaldi
  * output function OID.  If the returned category is JSONTYPE_CAST, we
  * return the OID of the type->JSON cast function instead.
  */
+#if POSTGIS_PGSQL_VERSION < 180
 static void
 json_categorize_type(Oid typoid,
 					 JsonTypeCategory *tcategory,
@@ -315,7 +337,7 @@ json_categorize_type(Oid typoid,
 			break;
 	}
 }
-
+#endif
 /*
  * Turn a Datum into JSON text, appending the string to "result".
  *
